#include <WiFi.h>
#include "SoftwareSerial.h"
#include <PubSubClient.h>
#include <Wire.h>

char ssid[] = "";      // your network SSID (name)
char pass[] = "";  // your network password

int status  = WL_IDLE_STATUS;    // the Wifi radio's status
WiFiClient wifiClient;  
PubSubClient client(wifiClient);  

char MqttServer[]     = "iiot.ideaschain.com.tw";  // new ideaschain dashboard MQTT server
int  MqttPort          = 1883;
char ClientId[]       = "";    // MQTT client ID. it's better to use unique id.
char Username[]       = "";    // device access token
char Password[]       = "";    // no need password
char SubscribeTopic[] = "v1/devices/me/telemetry";  // Fixed topic. ***DO NOT MODIFY***
char PublishTopic[]   = "v1/devices/me/telemetry";  // Fixed topic. ***DO NOT MODIFY***
char SubscribeTopic2[] = "v1/devices/me/attributes"; // Fixed topic. ***DO NOT MODIFY***
char PublishTopic2[]   = "v1/devices/me/attributes";  // Fixed topic. ***DO NOT MODIFY***
char PublishPayload[]="{\"weight\":\"0\"}"; // use to test

///
/// RGB LED
///
#include "ws2812b.h"
#define DIGITALPINNUMBER  10
#define NUM_LEDS  8
ws2812b Ledstrip = ws2812b(DIGITALPINNUMBER , NUM_LEDS);

//
// OLED 128x128
//
#include <U8g2lib.h>
U8G2_SSD1327_MIDAS_128X128_F_HW_I2C u8g2(U8G2_R0,  U8X8_PIN_NONE);


///
/// HX711重量模組
///
#include "HX711.h"
const int DT_PIN = 6;
const int SCK_PIN = 2;
const int SCALE_FACTOR = -20; //the propotional parameter,different from each person
HX711 Scale;

void reconnect();

void setup(void) {
  Serial.begin(9600);
 
  u8g2.begin();  
  u8g2.clear(); //clear cache in OLED
  u8g2.setFont(u8g2_font_ncenB08_tr); //set the font
  u8g2.firstPage();
  do {
     u8g2.setCursor(30 , 64);
     u8g2.print("connecting...");    
  } while ( u8g2.nextPage() );
  
  while (status != WL_CONNECTED) {
    Serial.print("Attempting to connect to SSID: ");
    Serial.println(ssid);
    // Connect to WPA/WPA2 network. Change this line if using open or WEP network:
    status = WiFi.begin(ssid, pass);
    // wait 4 seconds for connection:
    delay(4000);
  }
  
  u8g2.clear();
  u8g2.firstPage();
  do {
    u8g2.setCursor(17, 70);
    u8g2.print("connect successed!");    
  } while ( u8g2.nextPage() );
  
  client.setServer(MqttServer, MqttPort); 
  reconnect();
  
  client.setCallback(callback); 
  if (client.connect(Username)){
    Serial.println("Connection has been established, well done");
  } 
  
  Scale.begin(DT_PIN, SCK_PIN);
  Scale.set_scale(SCALE_FACTOR);//set the propotional parameter 設定比例參數
  Scale.tare();//  zero weight sensor 
   
  delay(1000);
  u8g2.clear();
  u8g2.firstPage();
  do {
    u8g2.setCursor(13, 64);
    u8g2.print("starting operating"); 
  } while ( u8g2.nextPage() );
  Ledstrip.begin(); //RGB begin
  delay(1500);
}

float Measure_Weight = 0;
float Weight = 0;
float Number = 0;
float SumWeight = 0 ;
bool WhetherContinue = false;
bool Next = false ;
String Str = "\0";  //use to store the message you want to display on OLED
unsigned long TimeNow = 0;
unsigned long TimeNow1 = 0;
unsigned long TimeNow2 = 0;
unsigned long TimeNow3 = 0;
unsigned long DisplayTime = 0;

void loop(void) {
  client.loop();// to keep client not be interrupt by other running program,you can switch to compare function by upload mqtt payload format( {"weight":" ","number":" "}
  String payload = "\0";
  Measure_Weight = Scale.get_units(10); //measure the weight 10 times and get the average
  if(Measure_Weight < 0)Measure_Weight = 0;
  Serial.println("Measure_Weight: ");
  Serial.println(Measure_Weight);
  payload = "{\"Measure_Weight\":\""+String(Measure_Weight) +"\"}";//basic format to upload data 
  
  if(Measure_Weight < 30 &&   millis()- TimeNow > 30000){  //haven't put anything to weigh,every 30 seconds upload
    client.publish(PublishTopic, payload.c_str());
    TimeNow = millis();
  }
  else if(Measure_Weight > 30 &&   millis()- TimeNow > 5000){ //have put anything to weigh,every 5 seconds upload
    client.publish(PublishTopic, payload.c_str());
    TimeNow = millis();
  }
  if(millis()- DisplayTime > 1500){ //every 1.5 second display on OLED
    Str = "measure weight : ";
    Str = Str + String(Measure_Weight);
    u8g2.clear();
    u8g2.firstPage();
    do {
    u8g2.setCursor(10, 64);
    u8g2.print(Str);   //display the measure_weight message
    } while ( u8g2.nextPage() );
    DisplayTime = millis();
  }
  
  ///
  /// if there isn't any message sended to the topic ,it'll keep weighing (interrupted by the payload {"continue":""} )
  ///
  while(WhetherContinue){ 
    client.loop();// to keep client not be interrupt by other running program
    
    if(millis()- TimeNow1 > 3000 && millis() > TimeNow1 ){  //  3 seconds  after  TimeNow3 program done ,starting to run
      Measure_Weight = Scale.get_units(10); //measure the weight
   // Serial.println(Measure_Weight);
      if(Measure_Weight < 0) Measure_Weight = 0 ; // it might weigh negative value when you put nothing on
        Str = "total weight : ";
        Str = Str + String(Measure_Weight);
      u8g2.clear();
      u8g2.firstPage();
      do {
        u8g2.setCursor(10, 64);
        u8g2.print(Str);   //display the measure_weight message
      } while ( u8g2.nextPage() );
      TimeNow1 = 12000.0 + millis() ; //avoid collison with another time
      TimeNow3 = 12000.0 + millis() ; //avoid collison with another time
      TimeNow2 = millis(); 
    }
    if(millis()- TimeNow2 > 3000 && millis() > TimeNow2 ){ //  3 seconds  after  TimeNow1 program done ,starting to run
      String payload_fail = "\0";
      String payload2_pass = "\0";
      /*
      Serial.println(SumWeight - Measure_Weight);
      Serial.println(Measure_Weight*0.05  );
      */
      if( fabs(SumWeight - Measure_Weight)> SumWeight*0.05 ){ //if measure_weight smaller or bigger than 0.95 time total_weight
        int need_number = fabs(SumWeight - Measure_Weight)/Weight ; //to calculate the elements you need to increase or decrease
        if(need_number == 0)need_number = 1; //cuz using division to calculate ,it milght compute 0 in extreme condition 
        ///
        ///the failed payload format to IdeasChain and MQTT Dash to get the relevant value
        ///
        payload_fail="{\"Measure_Weight\":\""+ String(Measure_Weight) +"\",\"weight\":\"" + String(Weight) + "\",\"number\":\"" + String(Number) + "\",\"Y/N\":\"N\",\"color\":\"#ff0000\",\"need\":\""+String(need_number)+"\"}";
        Serial.println(payload_fail);
        client.publish(PublishTopic, payload_fail.c_str());
        // Serial.println(need_number);
        Str = "need  ";
        Str  = Str + String(need_number);
        u8g2.clear();
        if(SumWeight - Measure_Weight < 0){ //measure_weight smaller than 0.95 total_weigh (you should increase elements to pass)
          u8g2.firstPage();
          do {
            u8g2.setCursor(50, 40);
            u8g2.print("fail!!");
            u8g2.setCursor(46, 60);
            u8g2.print(Str); //display the elements you need to increase message
            u8g2.setCursor(25, 80);
            u8g2.print("more elements");   
          } while ( u8g2.nextPage() );
        }
        else{ //measure_weight bigger than 0.95 total_weigh (you should decrease elements to pass)
          do {
            u8g2.setCursor(50, 40);
            u8g2.print("fail!!");
            u8g2.setCursor(46, 60);
            u8g2.print(Str);  //display the elements you need decrease message
            u8g2.setCursor(25, 80);
            u8g2.print("less elements");   
          } while ( u8g2.nextPage() );
        }
        for(int i = 0 ; i < 8 ; i++){
          Ledstrip.setPixelColor(i,255,0,0); // All_RGB_to_Red
        }
        Ledstrip.show();
      }
    
      else{
        ///
        ///the pass payload format to IdeasChain and MQTT Dash to get the relevant value
        ///
        payload2_pass="{\"Measure_Weight\":\""+ String(Measure_Weight) +"\",\"weight\":\"" + String(Weight) + "\",\"number\":\"" + String(Number) + "\",\"Y/N\":\"Y\",\"color\":\"#00ff11\"}";
        Serial.println(payload2_pass);
        client.publish(PublishTopic, payload2_pass.c_str());
        u8g2.firstPage();
        do {
          u8g2.setCursor(35, 64);
          u8g2.print("success!!");
        } while ( u8g2.nextPage() );
    
        for(int i = 0 ; i < 8 ; i++){
          Ledstrip.setPixelColor(i,0,255,0); //All_RGB_to_Green
        }
        Ledstrip.show();
      }
      TimeNow1 = 12000.0 + millis() ; //avoid collison with another time
      TimeNow2 = 12000.0 + millis() ; //avoid collison with another time
      TimeNow3 = millis();
    }
    if(millis()- TimeNow3 > 3000 && millis() > TimeNow3 ){ //  3 seconds  after  TimeNow2 program done ,starting to run
      u8g2.clear();
      u8g2.firstPage();
      do {
        u8g2.setCursor(10, 64);
        u8g2.print("continue comparing");    
      } while ( u8g2.nextPage() );
      for(int i = 0 ; i < 8 ; i++){
        Ledstrip.setPixelColor(i,0,0,0); //stop RGB 
      }
      Ledstrip.show();
      Next = true;
      TimeNow2 = 12000.0 + millis() ;  //avoid collison with another time
      TimeNow3 = 12000.0 + millis() ;  //avoid collison with another time
      TimeNow1 = millis(); 
    }
  }    
  ///
  /// set up to next goods ,wating to enter the payload (weight and number)
  ///  
  if(Next){
   u8g2.clear();
   u8g2.firstPage();
   do {
     u8g2.setCursor(0, 64);
     u8g2.print("switch to basic weigh"); 
   } while ( u8g2.nextPage() );
   for(int i = 0 ; i < 8 ; i++){
     Ledstrip.setPixelColor(i,0,0,0); //stop RGB  
   }
   Ledstrip.show();
   Next = false;
   delay(1500);
  }
  if(!client.connected()){
     reconnect();
     delay(2000);
  }
}

void callback(char* topic, byte* payload, unsigned int length) {
  ///
  /// cuz pulish to the same portal,using first word of playoad to determine what to do  ( the payload {"weight":" ","number":" "} is the format to start weighing)
  ///
  if(char(payload[2])!= 'w'){
    if (char(payload[2])== 'c')WhetherContinue = false ; 
    for (int i = 0; i < length; i++) {
      Serial.print((char)payload[i]);
      for(int i = 0 ; i < 8 ; i++){
        Ledstrip.setPixelColor(i,0,0,0); //stop RGB  
      }
    Ledstrip.show();
    }
    return;
  }
  /*
  Serial.print("Message arrived [");
  Serial.print(topic);
  Serial.print("] ");
  */
  int cnt = 0 ; //to point the position of payload
  String weight_str =""; 
  String number_str ="";
  ///
  ///get thw weight and number you sended
  ///
  for (int i = 0; i < length; i++) {
    // Serial.print((char)payload[i]);
    if((char)payload[i]=='\"') cnt ++ ;
    else if(cnt==7) weight_str = weight_str + (char)payload[i]; // position of two parameter
    else if(cnt==15) number_str = number_str + (char)payload[i];
  }
  Serial.println();
  Weight = atof(weight_str.c_str());
  Number = atof(number_str.c_str());
  SumWeight =  Weight * Number; //count the total weight
  u8g2.clear();
  u8g2.firstPage();
  do {
    u8g2.setCursor(20, 64);
    u8g2.print("start comparing");    
  } while ( u8g2.nextPage() );
  WhetherContinue = true; //set up continue measure
  delay(3000);
  TimeNow = millis(); //initialize time
  TimeNow1 = millis(); //initialize time
  TimeNow2 = millis()+10000; //avoid collison with another time
  TimeNow3 = millis()+10000; //avoid collison with another time
}


 
void reconnect() {
  // Loop until we're reconnected
  while (!client.connected()) {
    Serial.print("\nAttempting MQTT connection...");
    // Attempt to connect
    if (client.connect(ClientId,Username,Password)) {
      Serial.println("connected");
      // Once connected, publish an announcement...
      client.publish(PublishTopic2, PublishPayload);
      // ... and resubscribe
      client.subscribe(SubscribeTopic);
    } else {
      Serial.print("failed, rc=");
      Serial.print(client.state());
      Serial.println(" try again in 5 seconds");
      // Wait 5 seconds before retrying
      delay(5000);
    }
  }
}
